/************************************************************

Created: Thursday, October 19, 1989 at 11:10 PM
	PPCToolBox.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc. 1989-90
	All rights reserved

************************************************************/

#ifndef __PPCTOOLBOX__
#define __PPCTOOLBOX__

#ifndef __APPLETALK__
#include <AppleTalk.h>
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __TYPES__
#include <Types.h>
#endif


/*  service Type  */

enum {ppcServiceRealTime = 1};

/*  lookup Type  */

enum {ppcNoLocation = 0};                   /* There is no PPCLocName */
enum {ppcNBPLocation = 1};                  /* Use AppleTalk NBP      */
enum {ppcNBPTypeLocation = 2};              /* Used for specifying a location name type during PPCOpen only */

/*  port Types  */

enum {ppcByCreatorAndType = 1};             /* PortType is specified as colloquial Mac creator and type */
enum {ppcByString = 2};                     /* Port Type is in pascal string format */

/*  Values returned for requestType field in PPCInform call */

enum {ppcLocalOrigin = 1};   				/* session originated from this machine */
enum {ppcRemoteOrigin = 2};  				/* session originated from remote machine */

/*  ------------------------------------- 
 -900 to -950 are reserved for PPC     */

enum {notInitErr = -900};
enum {tooManyPortsErr = -901};
enum {nameTypeErr = -902};
enum {noPortErr = -903};
enum {noGlobalsErr = -904};
enum {localOnlyErr = -905};
enum {destPortErr = -906};
enum {sessTableErr = -907};
enum {noSessionErr = -908};
enum {badReqErr = -909};
enum {portNameExistsErr = -910};
enum {noPortTableErr = -911};
enum {userRejectErr = -912};
enum {noMachineNameErr = -913};
enum {noToolboxNameErr = -914};
enum {noResponseErr = -915};
enum {portClosedErr = -916};
enum {sessClosedErr = -917};
enum {atalkPhaseErr = -918};            /* apple talk is not phase 2 */
enum {badPortNameErr = -919};
enum {loginCancelErr = -920};
enum {unKnownUserErr = -921};
enum {noDefaultUserErr = -922};
enum {notLoggedInErr = -923};
enum {noUserRefErr = -924};
enum {networkErr = -925};
enum {noInformErr = -926};
enum {authFailErr = -927};
enum {noUserRecErr = -928};
enum {notSupportedErr = -929};
enum {badServiceMethodErr = -930};
enum {badLocNameErr = -931};
enum {guestNotAllowedErr = -932};

typedef	unsigned char	PPCServiceType;
typedef	short	PPCLocationKind;
typedef	short	PPCPortKinds;
typedef	unsigned char	PPCSessionOrigin; 	/* Values returned for request field in PPCInform call */
typedef	short	PPCPortRefNum;	
typedef	long	PPCSessRefNum;	
/* Procedures you will need to write */
/* ex:  void 			MyCompletionRoutine(PPCParamBlkPtr pb) */
/* ex:  pascal Boolean	MyPortFilter(LocationNamePtr locationName, PortInfoPtr thePortInfo) */
typedef ProcPtr PPCCompProcPtr;	

#define PPCHeader \
	 Ptr			   qLink;			/* PPC's Internal Use */\
	 unsigned short    csCode;			/* Requested PPC command */\
	 unsigned short    intUse;			/* Internal Use */\
	 Ptr			   intUsePtr;		/* Internal Use */\
	 PPCCompProcPtr	   ioCompletion;	/* Completion Routine	*/\
	 OSErr			   ioResult;		/* Command Result Code */\
	 unsigned long	   Reserved[5];		/* Reserved for PPC, Don't use */


struct PPCPortRec
{
	 ScriptCode	 	nameScript;      	/* script of name */
	 Str32			name;     	    	/* name of port as seen in browser */
	 PPCPortKinds	portKindSelector;   /* which variant */
	 union
	{
		 Str32		portTypeStr;         
		 struct
		{
			 OSType	creator;
			 OSType	type;
		} port; 					
	} u;
};

typedef struct PPCPortRec PPCPortRec;
typedef PPCPortRec *PPCPortPtr;

struct LocationNameRec
{
	PPCLocationKind locationKindSelector;	/*  which variant */
	union
	{
		EntityName	nbpEntity;          	/* NBP name entity */
		Str32 		nbpType;				/* just the NBP type string, for PPCOpen */
	} u;
};

typedef struct LocationNameRec LocationNameRec;
typedef LocationNameRec *LocationNamePtr;


struct PortInfoRec
{
	unsigned char	filler1;
	Boolean 		authRequired;
	PPCPortRec		name;
};	
typedef struct PortInfoRec PortInfoRec;
typedef PortInfoRec *PortInfoPtr;
typedef PortInfoRec *PortInfoArrayPtr;
	
typedef pascal Boolean (*PPCFilterProcPtr)(LocationNamePtr locationName, PortInfoPtr thePortInfo);

struct PPCOpenPBRec
{
	PPCHeader
	/* 38 <--  */	PPCPortRefNum	portRefNum;			/*  Port Reference											*/
					long			filler1;			
	/* 44 -->  */	PPCServiceType	serviceType;		/*  Bit field describing the requested port service			*/
	/* 45 -->  */	unsigned char	resFlag;			/*  Must be set to 0 										*/       
	/* 46 -->  */	PPCPortPtr		portName;			/*  PortName for PPC 										*/
	/* 50 -->  */	LocationNamePtr	locationName;		/*  If NBP Registration is required 						*/
	/* 54 -->  */	Boolean			networkVisible;		/*  make this network visible on network 					*/
	/* 55 <--  */	Boolean			nbpRegistered;      /*  The given location name was registered on the network	*/
};
typedef struct PPCOpenPBRec PPCOpenPBRec;
typedef PPCOpenPBRec *PPCOpenPBPtr;


struct PPCInformPBRec
{
	PPCHeader
	/* 38 -->  */	PPCPortRefNum		portRefNum;			/* Port Reference										*/
	/* 40 <--  */	PPCSessRefNum 		sessRefNum;			/* Session Reference 									*/
	/* 44 <--  */	PPCServiceType		serviceType;		/* Status Flags for type of session: RealTime			*/ 
	/* 45 -->  */	Boolean				autoAccept;			/* if true session will be accepted automatically 		*/
	/* 46 -->  */	PPCPortPtr			portName;			/* Buffer for Source PPCPortRec 						*/ 
	/* 50 -->  */	LocationNamePtr		locationName;		/* Buffer for Source LocationNameRec 					*/ 
	/* 54 -->  */	StringPtr			userName;			/* Buffer for Soure user's name trying to link. 		*/
	/* 58 <--  */	unsigned long		userData;			/* value included in PPCStart's userData 				*/
	/* 62 <--  */	PPCSessionOrigin	requestType;		/* Local or Network										*/
};
typedef struct PPCInformPBRec PPCInformPBRec;
typedef PPCInformPBRec *PPCInformPBPtr;



struct PPCStartPBRec
{
	PPCHeader
	/* 38 -->  */	PPCPortRefNum	portRefNum;			/* Port Reference										*/
	/* 40 <--  */	PPCSessRefNum	sessRefNum;			/* Session Reference 									*/
	/* 44 <--  */	PPCServiceType	serviceType;		/* Actual service method (realTime)						*/
	/* 45 -->  */	unsigned char	resFlag;			/* Must be set to 0 									*/
	/* 46 -->  */	PPCPortPtr		portName;			/* Destination portName 								*/ 
	/* 50 -->  */	LocationNamePtr	locationName;		/* NBP style service location name 						*/ 
	/* 54 <--  */	unsigned long	rejectInfo;			/* reason for rejecting the session request 			*/
	/* 58 -->  */	unsigned long	userData;			/* Copied to destination PPCInform parameter block 		*/
	/* 62 -->  */	unsigned long	userRefNum;			/* userRefNum (obtained during login process) 			*/ 
};
typedef struct PPCStartPBRec PPCStartPBRec;
typedef PPCStartPBRec *PPCStartPBPtr;



struct PPCAcceptPBRec
{
	PPCHeader
					short 			filler1;           
	/* 40 -->  */	PPCSessRefNum	sessRefNum;          /* Session Reference 									*/ 
};
typedef struct PPCAcceptPBRec PPCAcceptPBRec;
typedef PPCAcceptPBRec *PPCAcceptPBPtr;



struct PPCRejectPBRec
{
	PPCHeader
					short			filler1;		
	/* 40 -->  */	PPCSessRefNum	sessRefNum;			/* Session Idenifier */
					short			filler2;		
					long			filler3;		
					long			filler4;		
	/* 54 -->  */	unsigned long	rejectInfo;			/* Copied to Source PPCStart parameter block */
};
typedef struct PPCRejectPBRec PPCRejectPBRec;
typedef PPCRejectPBRec *PPCRejectPBPtr;



struct PPCWritePBRec
{
	PPCHeader
					short 			filler1;			
	/* 40 -->  */	PPCSessRefNum	sessRefNum;			/* Session Reference 							*/
	/* 44 -->  */	Size		 	bufferLength;		/* Length of the message buffer 				*/ 
	/* 48 <--  */	Size		 	actualLength;		/* Actual Length Written 						*/ 
	/* 52 -->  */	Ptr 			bufferPtr;			/* Pointer to message buffer 					*/ 
	/* 56 -->  */	Boolean 		more;				/* if more data in this block will be written 	*/
					unsigned char 	filler2;			
	/* 58 -->  */	unsigned long 	userData;			/* Message block userData Uninterpreted by PPC	*/ 
	/* 62 -->  */	OSType		 	blockCreator;		/* Message block creator Uninterpreted by PPC 	*/ 
	/* 66 -->  */	OSType		 	blockType;			/* Messgae block Type Uninterpreted by PPC 		*/ 
};
typedef struct PPCWritePBRec PPCWritePBRec;
typedef PPCWritePBRec *PPCWritePBPtr;



struct PPCReadPBRec
{
	PPCHeader
					short 			filler1;			
	/* 40 -->  */	PPCSessRefNum	sessRefNum;			/* Session Reference 							*/
	/* 44 -->  */	Size		 	bufferLength;		/* Length of the message buffer 				*/ 
	/* 48 <--  */	Size		 	actualLength;		/* Actual Length read	 						*/ 
	/* 52 -->  */	Ptr 			bufferPtr;			/* Pointer to message buffer 					*/ 
	/* 56 <--  */	Boolean 		more;				/* if true more data in this block to be read 	*/
					unsigned char 	filler2;			
	/* 58 <--  */	unsigned long 	userData;			/* Message block userData Uninterpreted by PPC	*/ 
	/* 62 <--  */	OSType		 	blockCreator;		/* Message block creator Uninterpreted by PPC 	*/ 
	/* 66 <--  */	OSType		 	blockType;			/* Messgae block Type Uninterpreted by PPC 		*/ 
};
typedef struct PPCReadPBRec PPCReadPBRec;
typedef PPCReadPBRec *PPCReadPBPtr;



struct PPCEndPBRec
{
	PPCHeader
					short 			filler1;			
	/* 40 -->  */	PPCSessRefNum	sessRefNum;			/* Session Reference to end						*/
};
typedef struct PPCEndPBRec PPCEndPBRec;
typedef PPCEndPBRec *PPCEndPBPtr;



struct PPCClosePBRec
{
	PPCHeader
	/* 38 -->  */	PPCPortRefNum	portRefNum;			/* Port Reference to be closed					*/
};
typedef struct PPCClosePBRec PPCClosePBRec;
typedef PPCClosePBRec *PPCClosePBPtr;



struct IPCListPortsPBRec
{
	 PPCHeader
					short				filler1;			
	/* 40 -->  */	unsigned short		startIndex;     	/* Start Index 													*/ 
	/* 42 -->  */	unsigned short		requestCount;   	/* Number of entries to be returned 							*/
	/* 44 <--  */	unsigned short		actualCount;   		/* Actual Number of entries to be returned 						*/ 
	/* 46 -->  */	PPCPortPtr    		portName;			/* PortName Match 												*/ 
	/* 50 -->  */	LocationNamePtr		locationName;		/* NBP name to locate the Port Location 						*/ 
	/* 54 -->  */	PortInfoArrayPtr	bufferPtr;			/* Pointer to a buffer requestCount*sizeof(PortInfo) bytes big	*/ 
};
typedef struct IPCListPortsPBRec IPCListPortsPBRec;
typedef IPCListPortsPBRec *IPCListPortsPBPtr;


union PPCParamBlockRec
{
		PPCOpenPBRec		openParam;
		PPCInformPBRec 	  	informParam;
		PPCStartPBRec		startParam;
		PPCAcceptPBRec 		acceptParam;
		PPCRejectPBRec 		rejectParam;
		PPCWritePBRec		writeParam;
		PPCReadPBRec		readParam;
		PPCEndPBRec		  	endParam;
		PPCClosePBRec		closeParam;
		IPCListPortsPBRec	listPortsParam;
};
typedef union PPCParamBlockRec PPCParamBlockRec;
typedef  PPCParamBlockRec  *PPCParamBlockPtr;

#ifdef __cplusplus
extern "C" {
#endif

/* PPC Calling Conventions */

pascal OSErr PPCInit(void)
	= {0x7000,0xA0DD};

pascal OSErr PPCOpen(PPCOpenPBPtr PB, Boolean async);
#pragma parameter __D0 PPCOpenSync(__A0)
pascal OSErr PPCOpenSync(PPCOpenPBPtr PB)
	= {0x7001,0xA0DD};
#pragma parameter __D0 PPCOpenAsync(__A0)
pascal OSErr PPCOpenAsync(PPCOpenPBPtr PB)
	= {0x7001,0xA4DD};

pascal OSErr PPCInform(PPCInformPBPtr PB, Boolean async);
#pragma parameter __D0 PPCInformSync(__A0)
pascal OSErr PPCInformSync(PPCInformPBPtr PB)
	= {0x7003,0xA0DD};
#pragma parameter __D0 PPCInformAsync(__A0)
pascal OSErr PPCInformAsync(PPCInformPBPtr PB)
	= {0x7003,0xA4DD};

pascal OSErr PPCStart(PPCStartPBPtr PB, Boolean async);
#pragma parameter __D0 PPCStartSync(__A0)
pascal OSErr PPCStartSync(PPCStartPBPtr PB)
	= {0x7002,0xA0DD};
#pragma parameter __D0 PPCStartAsync(__A0)
pascal OSErr PPCStartAsync(PPCStartPBPtr PB)
	= {0x7002,0xA4DD};

pascal OSErr PPCAccept(PPCAcceptPBPtr PB, Boolean async);
#pragma parameter __D0 PPCAcceptSync(__A0)
pascal OSErr PPCAcceptSync(PPCAcceptPBPtr PB)
	= {0x7004,0xA0DD};
#pragma parameter __D0 PPCAcceptAsync(__A0)
pascal OSErr PPCAcceptAsync(PPCAcceptPBPtr PB)
	= {0x7004,0xA4DD};

pascal OSErr PPCReject(PPCRejectPBPtr PB, Boolean async);
#pragma parameter __D0 PPCRejectSync(__A0)
pascal OSErr PPCRejectSync(PPCRejectPBPtr PB)
	= {0x7005,0xA0DD};
#pragma parameter __D0 PPCRejectAsync(__A0)
pascal OSErr PPCRejectAsync(PPCRejectPBPtr PB)
	= {0x7005,0xA4DD};

pascal OSErr PPCWrite(PPCWritePBPtr PB, Boolean async);
#pragma parameter __D0 PPCWriteSync(__A0)
pascal OSErr PPCWriteSync(PPCWritePBPtr PB)
	= {0x7006,0xA0DD};
#pragma parameter __D0 PPCWriteAsync(__A0)
pascal OSErr PPCWriteAsync(PPCWritePBPtr PB)
	= {0x7006,0xA4DD};

pascal OSErr PPCRead(PPCReadPBPtr PB, Boolean async);
#pragma parameter __D0 PPCReadSync(__A0)
pascal OSErr PPCReadSync(PPCReadPBPtr PB)
	= {0x7007,0xA0DD};
#pragma parameter __D0 PPCReadAsync(__A0)
pascal OSErr PPCReadAsync(PPCReadPBPtr PB)
	= {0x7007,0xA4DD};

pascal OSErr PPCEnd(PPCEndPBPtr PB, Boolean async);
#pragma parameter __D0 PPCEndSync(__A0)
pascal OSErr PPCEndSync(PPCEndPBPtr PB)
	= {0x7008,0xA0DD};
#pragma parameter __D0 PPCEndAsync(__A0)
pascal OSErr PPCEndAsync(PPCEndPBPtr PB)
	= {0x7008,0xA4DD};

pascal OSErr PPCClose(PPCClosePBPtr PB, Boolean async);
#pragma parameter __D0 PPCCloseSync(__A0)
pascal OSErr PPCCloseSync(PPCClosePBPtr PB)
	= {0x7009,0xA0DD};
#pragma parameter __D0 PPCCloseAsync(__A0)
pascal OSErr PPCCloseAsync(PPCClosePBPtr PB)
	= {0x7009,0xA4DD};

pascal OSErr IPCListPorts(IPCListPortsPBPtr PB, Boolean async);
#pragma parameter __D0 IPCListPortsSync(__A0)
pascal OSErr IPCListPortsSync(IPCListPortsPBPtr PB)
	= {0x700A,0xA0DD};
#pragma parameter __D0 IPCListPortsAsync(__A0)
pascal OSErr IPCListPortsAsync(IPCListPortsPBPtr PB)
	= {0x700A,0xA4DD};

pascal OSErr DeleteUserIdentity ( unsigned long userRef);

pascal OSErr GetDefaultUser ( unsigned long *userRef,
							  StringPtr userName);

pascal OSErr StartSecureSession ( PPCStartPBPtr startPB,
								  StringPtr userName,
								  Boolean useDefault,
								  Boolean allowGuest,
								  Boolean *guestSelected,
								  StringPtr prompt);

pascal OSErr PPCBrowser ( const Str255 prompt,
						  const Str255 applListLabel,
						  Boolean defaultSpecified,
						  LocationNameRec 	*theLocation,
						  PortInfoRec 		*thePortInfo,
						  PPCFilterProcPtr	portFilter,
						  const Str32 		theLocNBPType)
						  = {0x303C, 0x0D00, 0xA82B};

/* competion Routine should be declared as
   void completionRoutine(PPCParamBlkPtr PB)
*/

#ifdef __cplusplus
}
#endif

#endif
