/************************************************************

Created: Friday, October 20, 1989 at 3:07 PM
	Sound.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc.	1986-1990
	All rights reserved

************************************************************/


#ifndef __SOUND__
#define __SOUND__

#ifndef __TYPES__
#include <Types.h>
#endif

#define swMode 					-1			/* Sound Driver modes */
#define ftMode 					 1
#define ffMode 					 0

#define synthCodeRsrc 			'snth'		/* Resource types used by Sound Manager */
#define soundListRsrc 			'snd '

#define twelfthRootTwo 			1.05946309434
#define infiniteTime 			0x7FFFFFFF

/* synthesizer numbers for SndNewChannel */

#define squareWaveSynth 		1            /*square wave synthesizer*/
#define waveTableSynth			3            /*wave table synthesizer*/
#define sampledSynth 			5            /*sampled sound synthesizer*/

/* command numbers for SndDoCommand and SndDoImmediate */

#define nullCmd					0
#define initCmd					1
#define freeCmd					2
#define quietCmd				3
#define flushCmd				4
#define reInitCmd				5

#define waitCmd					10
#define pauseCmd				11
#define resumeCmd				12
#define callBackCmd				13
#define syncCmd					14
#define emptyCmd				15

#define tickleCmd				20
#define requestNextCmd			21
#define howOftenCmd				22
#define wakeUpCmd				23
#define availableCmd			24
#define versionCmd				25
#define totalLoadCmd			26
#define loadCmd					27

#define scaleCmd				30
#define tempoCmd				31

#define freqDurationCmd			40
#define restCmd					41
#define freqCmd					42
#define ampCmd					43
#define timbreCmd				44

#define waveTableCmd			60
#define phaseCmd				61

#define soundCmd				80
#define bufferCmd				81
#define rateCmd					82
#define	continueCmd				83			
#define	doubleBufferCmd			84			
#define getRateCmd				85

#define	sizeCmd					90	
#define	convertCmd				91	

#define stdQLength				128
#define dataOffsetFlag			0x8000

#define waveInitChannelMask		0x07
#define waveInitChannel0		0x04
#define waveInitChannel1		0x05
#define waveInitChannel2		0x06
#define waveInitChannel3		0x07

/* channel initialization parameters */

#define initSRateMask			0x0030
#define initStereoMask			0x00C0
#define initChanLeft 			0x0002		/* left stereo channel */
#define initChanRight 			0x0003		/* right stereo channel */
#define initSRate22k 			0x0020		/* 22k sampling rate */
#define initMono 				0x0080		/* monophonic channel */
#define initStereo 				0x00C0		/* stereo channel */
#define	initNoInterp 			0x0004		/* no linear interpolation */
#define initNoSRC				0x0008		/* no sample rate conversion */
#define initMACE3				0x0300		/* MACE 3:1 */
#define initMACE6				0x0400		/* MACE 6:1 */

#define initChan0 				0x0004		/* channel 0 - wave table only */
#define initChan1 				0x0005		/* channel 1 - wave table only */
#define initChan2 				0x0006		/* channel 2 - wave table only */
#define initChan3 				0x0007		/* channel 3 - wave table only */

#define	stdSH					0x00		/* Standard sound header encode value */
#define	extSH					0xFF		/* Extended sound header encode value */
#define	cmpSH					0xFE		/* Compressed sound header encode value */

#define	notCompressed			0			/* compression ID's */
#define	twoToOne				1
#define	eightToThree			2
#define	threeToOne				3
#define	sixToOne				4

#define	outsideCmpSH			0
#define	insideCmpSH				1
#define	aceSuccess				0
#define	aceMemFull				1
#define	aceNilBlock				2
#define	aceBadComp				3
#define	aceBadEncode			4
#define	aceBadDest				5
#define	aceBadCmd				6
#define	sixToOnePacketSize		8	
#define	threeToOnePacketSize	16				
#define	stateBlockSize			64
#define	leftOverBlockSize		32

#define firstSoundFormat		0x0001		/* general sound format */
#define secondSoundFormat		0x0002		/* special sampled sound format (HyperCard) */

						
typedef long Time;							/* in half milliseconds */

typedef unsigned char Wave[256];
typedef Wave *WavePtr;

enum {sysBeepDisable = 0x0000};
enum {sysBeepEnable = 0x0001};
											/* unitTypes for AudioSelection.unitType */
enum {unitTypeNoSelection = 0xFFFF};
enum {unitTypeSeconds = 0x0000};

enum {sMgrBadParam = -213};					/* invalid argument */
enum {sMgrResNotFound = -214};

typedef unsigned char FreeWave[30001];

struct FFSynthRec {
	short mode;
	Fixed count;
	FreeWave waveBytes;
};

typedef struct FFSynthRec FFSynthRec;
typedef FFSynthRec *FFSynthPtr;

struct Tone {
	short count;
	short amplitude;
	short duration;
};

typedef struct Tone Tone;
typedef Tone Tones[5001];

struct SWSynthRec {
	short mode;
	Tones triplets;
};

typedef struct SWSynthRec SWSynthRec;
typedef SWSynthRec *SWSynthPtr;

struct FTSoundRec {
	short duration;
	Fixed sound1Rate;
	long sound1Phase;
	Fixed sound2Rate;
	long sound2Phase;
	Fixed sound3Rate;
	long sound3Phase;
	Fixed sound4Rate;
	long sound4Phase;
	WavePtr sound1Wave;
	WavePtr sound2Wave;
	WavePtr sound3Wave;
	WavePtr sound4Wave;
};

typedef struct FTSoundRec FTSoundRec;
typedef FTSoundRec *FTSndRecPtr;

struct FTSynthRec {
	short mode;
	FTSndRecPtr sndRec;
};

typedef struct FTSynthRec FTSynthRec;
typedef FTSynthRec *FTSynthPtr;

typedef struct SndCommand {
    unsigned short	cmd;
    short			param1;
    long			param2;
}SndCommand;

typedef struct SndChannel *SndChannelPtr;

typedef pascal void (*SndCompletionProcPtr)(void);
typedef pascal Boolean (*SndCallBackProcPtr)(SndChannelPtr chan, SndCommand cmd);

typedef struct ModifierStub ModifierStub;
typedef ModifierStub *ModifierStubPtr;

typedef pascal	Boolean	(*SndModifierProcPtr)(SndChannelPtr chan, SndCommand *cmd,
						ModifierStub *mod);

struct ModifierStub {
	struct ModifierStub	*nextStub;
	SndModifierProcPtr	code;
	long				userInfo;
	Time				count;
	Time				every;
	char				flags;
	char				hState;
};

struct SoundHeader {
	Ptr					samplePtr;			/* if NIL then samples are in sampleArea */
	unsigned long		length;				/* length of sound in bytes */
	Fixed				sampleRate;			/* sample rate for this sound */
	unsigned long		loopStart;			/* start of looping portion */
	unsigned long		loopEnd;			/* end of looping portion */
	unsigned char		encode;				/* header encoding */
	unsigned char		baseFrequency;		/* baseFrequency value */
    char sampleArea[1];
};

typedef struct SoundHeader SoundHeader;
typedef SoundHeader *SoundHeaderPtr;

struct SndChannel {
    struct SndChannel	*nextChan;
    ModifierStubPtr		firstMod;
    SndCallBackProcPtr	callBack;
    long				userInfo;
    Time				wait;			/* The following is for internal Sound Manager use only.*/
    SndCommand			cmdInProgress;
    short				flags;
    short				qLength;
    short				qHead;			/* next spot to read or -1 if empty */
    short				qTail;			/* next spot to write = qHead if full */
    SndCommand			queue[128];
};

typedef struct SndChannel SndChannel;

/*______________________ <hbs 1/10/89> structs for ACE____________________________ */


typedef struct{
	short						stateVar[stateBlockSize] ;
}   StateBlock, *StateBlockPtr;    


typedef struct 
{
	unsigned long				count;
	char						sampleArea[leftOverBlockSize];
}
LeftOverBlock, *LeftOverBlockPtr;


typedef struct
 {
	Ptr 				samplePtr;			/* if nil then samples are in sample area */
	unsigned long 	    numChannels;		/* CAFF, number of channels  mono = 1 */
	Fixed				sampleRate;			/* sample rate in Apples Fixed point representation */
	unsigned long 		loopStart;			/* loopStart of sound before compression */
	unsigned long 		loopEnd;			/* loopEnd of sound before compression */
	unsigned char 		encode;				/* data structure used ,  stdSH, extSH, or cmpSH */
	unsigned char 		baseFrequency;		/* same meaning as regular SoundHeader */
	unsigned long 		numFrames;			/* length in frames	( packetFrames or sampleFrames */
	extended 			AIFFSampleRate;		/* CAFF, IEEE sample rate */
	Ptr 				markerChunk;		/* CAFF, sync track */
	Ptr 				futureUse1;			/* reserved by Apple */
	Ptr				 	futureUse2;			/* reserved by Apple */
	StateBlockPtr 		stateVars;			/* pointer to State Block */	
	LeftOverBlockPtr 	leftOverSamples;	/* used to save truncated samples between compression calls */
	unsigned short		compressionID;		/* 0 means no compression, non zero means compressionID */
	unsigned short 	    packetSize;			/* CAFF, number of bits in compressed sample packet */	
	unsigned short 	    snthID;				/* Resource ID of Sound Manager snth that contains NRT C/E */
	unsigned short 	    sampleSize;			/* CAFF, number of bits in non-compressed sample */
	char 				sampleArea[1];		/* space for when samples follow directly						*/		
}	
CmpSoundHeader, *CmpSoundHeaderPtr;

typedef struct
 {
	Ptr 				samplePtr;			/* if nil then samples are in sample area */
	unsigned long 	    numChannels;		/* AIFF, number of channels,  ie mono = 1 */
	Fixed				sampleRate;			/* sample rate in Apples Fixed point representation */
	unsigned long 		loopStart;			/* same meaning as regular SoundHeader */	
	unsigned long 		loopEnd;			/* same meaning as regular SoundHeader */
	unsigned char 		encode;				/* data structure used ,  stdSH, extSH, or cmpSH */
	unsigned char 		baseFrequency;		/* same meaning as regular SoundHeader */
	unsigned long 		numFrames;			/* length in total number of frames */
	extended 			AIFFSampleRate;		/* AIFF, IEEE sample rate */
	Ptr 				markerChunk;		/* AIFF, sync track */
	Ptr 				instrumentChunks;	/* AIFF, */
	Ptr 				AESRecording;		/* AIFF, */
	unsigned short 	    sampleSize;			/* AIFF, number of bits in sample */
	unsigned short 		futureUse1;			/* reserved by Apple */	
	unsigned long 		futureUse2;			/* reserved by Apple */	
	unsigned long 		futureUse3;			/* reserved by Apple */	
	unsigned long 		futureUse4;			/* reserved by Apple */
	char 				sampleArea[1];		/* space for when samples follow directly						*/		
}	
ExtSoundHeader, *ExtSoundHeaderPtr;



typedef struct{
	short				smMaxCPULoad;
	short				smNumChannels;
	short				smCurCPULoad;
}
SMStatus, *SMStatusPtr;


typedef struct{ 
	Fixed				scStartTime;
	Fixed				scEndTime;
	Fixed				scCurrentTime;
	Boolean				scChannelBusy;
	Boolean				scChannelDisposed;
	Boolean				scChannelPaused;
	char				scUnused;
	unsigned long		scChannelAttributes;
	long				scCPULoad;
}
SCStatus,*SCStatusPtr;


typedef struct {
	long				dbNumFrames;
	long				dbFlags;
	long				dbUserInfo[2];
	char				dbSoundData[1];
} SndDoubleBuffer, *SndDoubleBufferPtr;


typedef struct{ 
	long				unitType;
	Fixed				selStart;
	Fixed				selEnd;
}AudioSelection, *AudioSelectionPtr;


typedef struct {
	short				dbhNumChannels;
	short				dbhSampleSize;
	short				dbhCompressionID;
	short				dbhPacketSize;
	Fixed				dbhSampleRate;
	SndDoubleBufferPtr	dbhBufferPtr[2];
	ProcPtr				dbhDoubleBack;
}
SndDoubleBufferHeader, *SndDoubleBufferHeaderPtr;

						
enum {dbBufferReady = 0x00000001};
enum {dbLastBuffer = 0x00000004};

struct ModRef {
    unsigned short modNumber;
    long modInit;
};

typedef struct ModRef ModRef;

struct SndListResource {
	short format;
	short numModifiers;
	ModRef modifierPart[1]; 		/*This is a variable length array*/
	short numCommands;
	SndCommand commandPart[1];		/*This is a variable length array*/
	char dataPart[1];				/*This is a variable length array*/
};

typedef struct SndListResource SndListResource;
typedef SndListResource *SndListPtr;

#ifdef __cplusplus
extern "C" {
#endif
pascal OSErr SndDoCommand(SndChannelPtr chan,const SndCommand *cmd,Boolean noWait)
	= 0xA803;
pascal OSErr SndDoImmediate(SndChannelPtr chan,const SndCommand *cmd)
	= 0xA804;
pascal OSErr SndNewChannel(SndChannelPtr *chan,short synth,long init,SndCallBackProcPtr userRoutine)
	= 0xA807;
pascal OSErr SndDisposeChannel(SndChannelPtr chan,Boolean quietNow)
	= 0xA801;
pascal OSErr SndPlay(SndChannelPtr chan,Handle sndHdl,Boolean async)
	= 0xA805;
pascal OSErr SndControl(short id,SndCommand *cmd)
	= 0xA806;


/**/
/*
/*			New Enhanced Sound Manager 7.0 Calls
/*
/*			These will soon be supported by a Glue Library so that all this ugly
/*			shit will disappear and the traps will look like non-indexed traps.
/*
/**/

#define	MOVEL						0x203C	

/* constants for MACE in the dispatcher */
#define	MACEToolNum					16		
#define	sMgrMACEVersion				 0
#define sMgrComp3to1				 4
#define sMgrExp1to3					 8
#define sMgrComp6to1				12
#define sMgrExp1to6					16

/* constants for Sound Manager Extensions in the dispatcher */
#define sMgrToolNum					 8		
#define	sMgrSndStartFilePlay		3328		/* (13*256)+0 */
#define	sMgrSndPauseFilePlay		516			/* (2*256)+4 */
#define	sMgrSndStopFilePlay			776			/* (3*256)+8 */
#define	sMgrSndSoundManagerVersion	12
#define	sMgrSndChannelStatus		16
#define	sMgrSndManagerStatus		20
#define	sMgrSndGetSysBeepState		24
#define	sMgrSndSetSysBeepState		28
#define	sMgrSndPlayDoubleBuffer		32
#define	sMgrSndGetBufferStufferLoad	36
#define	sMgrSndGetMixerLoad			40

pascal unsigned long 
SndSoundManagerVersion (void)
	= {MOVEL,sMgrSndSoundManagerVersion,sMgrToolNum,0xA800};

pascal OSErr
SndStartFilePlay ( SndChannelPtr theChannel, short fRefNum,
			short resNum, long bufferSize, Ptr bufferPtr,
			AudioSelectionPtr theSelection, ProcPtr theCompletion, Boolean async)
	= {MOVEL,sMgrSndStartFilePlay,sMgrToolNum,0xA800};

pascal OSErr
SndPauseFilePlay (SndChannelPtr theChannel)
	= {MOVEL,sMgrSndPauseFilePlay,sMgrToolNum,0xA800};

pascal OSErr
SndStopFilePlay (SndChannelPtr theChannel, Boolean noWait)
	= {MOVEL,sMgrSndStopFilePlay,sMgrToolNum,0xA800};
	
pascal OSErr 
SndChannelStatus (SndChannelPtr theChannel, 
				short theLength, SCStatusPtr theStatus)
	= {MOVEL,sMgrSndChannelStatus,sMgrToolNum,0xA800};
	
pascal OSErr 
SndManagerStatus (short theLength, SMStatusPtr theStatus)
	= {MOVEL,sMgrSndManagerStatus,sMgrToolNum,0xA800};
	
pascal void 
SndGetSysBeepState ( short *sysBeepState)
	= {MOVEL,sMgrSndGetSysBeepState,sMgrToolNum,0xA800};

pascal OSErr 
SndSetSysBeepState (short sysBeepState)
	= {MOVEL,sMgrSndSetSysBeepState,sMgrToolNum,0xA800};

pascal OSErr
SndPlayDoubleBuffer ( SndChannelPtr theChannel, SndDoubleBufferHeaderPtr theParams)
	= {MOVEL,sMgrSndPlayDoubleBuffer,sMgrToolNum,0xA800};

pascal short 
SndGetBufferStufferLoad (short channelFeatures)
	= {MOVEL,sMgrSndGetBufferStufferLoad,sMgrToolNum,0xA800};

pascal short 
SndGetMixerLoad (short numChannels)
	= {MOVEL,sMgrSndGetMixerLoad,sMgrToolNum,0xA800};


pascal unsigned long
MACEVersion (void)
	= {MOVEL,sMgrMACEVersion,MACEToolNum,0xA800};

pascal void Comp3to1 ( Ptr inBuffer, Ptr outBuffer, unsigned long  Cnt, Ptr inState,
		Ptr outState, unsigned long  numChannels, unsigned long  whichChannel )
	= {MOVEL,sMgrComp3to1,MACEToolNum,0xA800};

pascal void Exp1to3 ( Ptr inBuffer, Ptr outBuffer, unsigned long  Cnt, Ptr inState, 
		Ptr outState, unsigned long  numChannels, unsigned long  whichChannel)
	= {MOVEL,sMgrExp1to3,MACEToolNum,0xA800};

pascal void Comp6to1 ( Ptr inBuffer, Ptr outBuffer, unsigned long  Cnt, Ptr inState, 
		Ptr outState, unsigned long  numChannels, unsigned long  whichChannel )
	= {MOVEL,sMgrComp6to1,MACEToolNum,0xA800};

pascal void Exp1to6 ( Ptr inBuffer, Ptr outBuffer, unsigned long  Cnt, Ptr inState,
		Ptr outState, unsigned long  numChannels, unsigned long  whichChannel)
	= {MOVEL,sMgrExp1to6,MACEToolNum,0xA800};


pascal void SetSoundVol(short level);
pascal void GetSoundVol(short *level);
pascal void StartSound(Ptr synthRec,long numBytes,SndCompletionProcPtr completionRtn);
pascal void StopSound(void);
pascal Boolean SoundDone(void); 
pascal OSErr SndAddModifier(SndChannelPtr chan,SndModifierProcPtr modifier,
    short id,long init)
    = 0xA802; 
#ifdef __cplusplus
}
#endif

#endif
