/************************************************************

Created: Wednesday, November 1, 1989
	AppleEvents.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc. 1985-1990
	All rights reserved

************************************************************/


#ifndef __APPLEEVENTS__
#define __APPLEEVENTS__

#ifndef __TYPES__
#include <Types.h>
#endif
#ifndef __MEMORY__
#include <Memory.h>
#endif
#ifndef __OSUTILS__
#include <OSUtils.h>
#endif
#ifndef __EVENTS__
#include <Events.h>
#endif
#ifndef __EPPC__
#include <EPPC.h>
#endif


#define typeLongInteger 'long'
#define typeShortInteger 'shor'
#define typeLongFloat 'doub'
#define typeShortFloat 'sing'
#define typeExtended 'exte'
#define typeComp 'comp'
#define typeMagnitude 'magn'
#define typeAEList 'list'
#define typeAERecord 'reco'
#define typeTrue 'true'
#define typeFalse 'fals'
#define typeAlias 'alis'
#define typeEnumerated 'enum'
#define typeType 'type'
#define typeAppParameters 'appa'
#define typeProperty 'prop'
#define typeFSS 'fss '
#define typeKeyword 'keyw'
#define typeSectionH 'sect'
#define typeWildCard '****'
#define typeApplSignature 'sign'
#define typeSessionID 'ssid'
#define typeTargetID 'targ'
#define typeProcessSerialNumber 'psn '
#define typeNull 'null'
#define kCoreEventClass 'aevt'
#define kAEOpenApplication 'oapp'
#define kAEOpenDocuments 'odoc'
#define kAEPrintDocuments 'pdoc'
#define kAEQuitApplication 'quit'
#define kAEAnswer 'ansr'
#define keyDirectObject '----'
#define keyPreDispatch 'phac'
#define keySelectProc 'selh'
#define keyTransactionIDAttr 'tran'
#define keyReturnIDAttr 'rtid'
#define keyEventClassAttr 'evcl'
#define keyEventIDAttr 'evid'
#define keyAddressAttr 'addr'
#define keyOptionalKeywordAttr 'optk'
#define keyTimeoutAttr 'timo'
#define keyInteractLevelAttr 'inte'
#define keyEventSourceAttr 'esrc'
#define keyMissedSourceAttr 'miss'
#define kAENoReply 0x00000001           /* Sender doesn't want a reply to event */
#define kAEQueueReply 0x00000002        /* Sender wants a reply but won't wait */
#define kAEWaitReply 0x00000003         /* Sender wants a reply and will be waiting */
#define kAENeverInteract 0x00000010     /* Server should not interact with user */
#define kAECanInteract 0x00000020       /* Server may try to interact with user */
#define kAEAlwaysInteract 0x00000030    /* Server should always interact with user */
#define kAECanSwitchLayer 0x00000040    /* Interaction may switch layer */
#define kAEDontReconnect 0x00000080     /* don't reconnect if there is a sessClosedErr */
#define kAEWantReceipt nReturnReceipt   /* Send wants a receipt of message */
#define kAENormalPriority 0x00000000    /* Post message at the end of event queue */
#define kAEHighPriority nAttnMsg        /* Post message at the front of the event queue */
#define kAnyTransactionID 0
#define kAutoGenerateReturnID -1
#define kAEDefaultTimeout -1
#define kNoTimeOut -2
#define kAENoDispatch 0
#define kAEUseStandardDispatch -1

/* Error messages in response to reading and writing buffer contents */

#define errAECoercionFail -1700
#define errAEDescNotFound -1701
#define errAECorruptData -1702
#define errAEWrongDataType -1703
#define errAENotAEDesc -1704
#define errAEBadListItem -1705
#define errAENewerVersion -1706
#define errAENotAppleEvent -1707

/* Error messages in response to sending/receiving a message */

#define errAEEventNotHandled -1708      /* The event isn't in the supplied event table */
#define errAEReplyNotValid -1709        /* AEResetTimer was passed an invalid reply */
#define errAEUnknownSendMode -1710      /* Mode wasn't NoReply, WaitReply, QueueReply; or Interaction level is unknown */
#define errAEWaitCanceled -1711         /* User cancelled out of wait loop for reply or receipt */
#define errAETimeout -1712              /* AppleEvent time out */
#define errAENoUserInteraction -1713    /* no user interaction allowed */
#define errAENotASpecialFunction -1714  /* there is no special function with this keyword */
#define errAEParamMissed -1715			/* a required parameter was not accessed */
#define errAEUnknownAddressType -1716   /* The target address type is not known */
#define errAEHandlerNotFound -1717      /* The event isn't in the supplied event table */
#define errAEReplyNotArrived -1718      /* the contents of the reply you are accessing have not arrived yet */
#define errAEIllegalIndex -1719      	/* the contents of the reply you are accessing have not arrived yet */

enum {kAEDataArray,kAEPackedArray,kAEHandleArray,kAEDescArray,kAEKeyDescArray};
typedef unsigned char AEArrayType;

enum {kAEInteractionWithSelf,kAEInteractWithLocal,kAEInteractWithAll};
typedef unsigned char AEInteractAllowed;

enum {kAEUnknownSource,kAEDirectCall,kAESameProcess,kAELocalProcess,kAERemoteProcess};
typedef unsigned char AEEventSource;


typedef unsigned long AEKeyword;

typedef unsigned long AEEventClass;

typedef unsigned long AEEventID;

typedef ResType DescType;


struct AEDesc {
	DescType descriptorType;
 	Handle dataHandle;
	} ;
typedef struct AEDesc AEDesc ;

typedef AEDesc			AEAddressDesc;
typedef AEDesc			AEDescList;
typedef AEDescList		AERecord;
typedef AERecord		AppleEvent;
	
typedef ProcPtr		EventHandlerProcPtr;
typedef ProcPtr		IdleProcPtr;
typedef ProcPtr		EventFilterProcPtr;

struct AEKeyDesc {
	AEKeyword descKey ;
	AEDesc descContent ;
	} ;
typedef struct AEKeyDesc AEKeyDesc ;

union AEArrayData {
	short AEDataArray[1];
	char AEPackedArray[1];
	Handle AEHandleArray[1];
	AEDesc AEDescArray[1];
	AEKeyDesc AEKeyDescArray[1];
	} ;
typedef union AEArrayData AEArrayData;
typedef AEArrayData *AEArrayDataPointer;

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************************/

/* The following calls apply to any AEDesc. Every result descriptor is created for you,
so you will be responsible for memory management of the descriptors so created.
Purgable descriptor data is not supported: the AEM does not call LoadResource. */

pascal OSErr AECreateDesc(DescType typeCode,Ptr dataPtr,Size dataSize,AEDesc *result)
	= {0x303C,0x0825,0xA816};

pascal OSErr AECoercePtr(DescType typeCode,Ptr dataPtr,Size dataSize,DescType toType,
	AEDesc *result)
	= {0x303C,0x0A02,0xA816};
	
pascal OSErr AECoerceDesc(AEDesc theAEDesc,DescType toType,AEDesc *result)
	= {0x303C,0x0603,0xA816};
	
pascal OSErr AEDisposeDesc(AEDesc *theAEDesc)
	= {0x303C,0x0204,0xA816};
	
pascal OSErr AEDuplicateDesc(AEDesc theAEDesc,AEDesc *result)
	= {0x303C,0x0405,0xA816};

/***********************************************************************/

/* The following calls apply to AEDescList.
  Since AEDescList is a subtype of AEDesc, the calls in the previous
  section can also be used for AEDescList. All list and array indices are 1-based.
  If the data was greater than maximumSize in the routines below, then actualSize will
  be greater than maximumSize, but only maximumSize bytes will actually be retrieved. */

pascal OSErr AECreateList(Ptr factoringPtr,Size factoredSize,Boolean isRecord, 
	AEDescList *resultList)
	= {0x303C,0x0706,0xA816};

pascal OSErr AECountItems(AEDescList theAEDescList,long *theCount)
	= {0x303C,0x0407,0xA816};

pascal OSErr AEPutPtr(AEDescList theAEDescList,long index,DescType typeCode,Ptr dataPtr,
	Size dataSize)
	= {0x303C,0x0A08,0xA816};

pascal OSErr AEPutDesc(AEDescList theAEDescList,long index,AEDesc theAEDesc)
	= {0x303C,0x0609,0xA816};

pascal OSErr AEGetNthPtr(AEDescList theAEDescList,long index,DescType desiredType,
	AEKeyword *theAEKeyword,DescType *typeCode,Ptr dataPtr,Size maximumSize,Size *actualSize)
	= {0x303C,0x100A,0xA816};

pascal OSErr AEGetNthDesc(AEDescList theAEDescList,long index,DescType desiredType,
	AEKeyword *theAEKeyword,AEDesc *theAEDesc)
	= {0x303C,0x0A0B,0xA816};

pascal OSErr AESizeOfNthItem(AEDescList theAEDescList,long index,DescType *typeCode,
	Size *dataSize)
	= {0x303C,0x082A,0xA816};

pascal OSErr AEGetArray(AEDescList theAEDescList,AEArrayType arrayType,AEArrayDataPointer arrayPtr,
	Size maximumSize,DescType *itemType,Size *itemSize,long *itemCount)
	= {0x303C,0x0D0C,0xA816};

pascal OSErr AEPutArray(AEDescList theAEDescList,AEArrayType arrayType,AEArrayDataPointer arrayPtr,
	DescType itemType,Size itemSize,long itemCount)
	= {0x303C,0x0B0D,0xA816};

pascal OSErr AEDeleteIndex(AEDescList theAEDescList,long index)
	= {0x303C,0x040E,0xA816};

/***********************************************************************/

/* The following calls apply to AERecord.
  Since AERecord is a subtype of AEDescList, the calls in the previous
  sections can also be used for AERecord
  an AERecord can be created by using AECreateList with isRecord set to true */

pascal OSErr AEPutKeyPtr(AERecord theAERecord,AEKeyword key,DescType typeCode,Ptr dataPtr,
	Size dataSize)
	= {0x303C,0x0A0F,0xA816};

pascal OSErr AEPutKeyDesc(AERecord theAERecord,AEKeyword key,AEDesc theDesc)
	= {0x303C,0x0610,0xA816};

pascal OSErr AEGetKeyPtr(AERecord theAERecord,AEKeyword key,DescType desiredType,
	DescType *typeCode,Ptr dataPtr,Size maximumSize,Size *actualSize)
	= {0x303C,0x0E11,0xA816};

pascal OSErr AEGetKeyDesc(AERecord theAERecord,AEKeyword key,DescType desiredType,
	AEDesc *result)
	= {0x303C,0x0812,0xA816};

pascal OSErr AESizeOfKeyDesc(AERecord theAERecord,AEKeyword key,DescType *typeCode,
	Size *dataSize)
	= {0x303C,0x0829,0xA816};

pascal OSErr AEDeleteKeyDesc(AERecord theAERecord,AEKeyword key)
	= {0x303C,0x0413,0xA816};

/*--------------------------------------------------------------------------------*/
/* The following calls are used to pack and unpack parameters from records of
type AppleEvent. Since AppleEvent is a subtype of AERecord, the calls in the previous
sections can also be used for AppleEvent. The next six calls are in fact identical
to the six calls for AERecord.  */

pascal OSErr AEPutParamPtr(AppleEvent theAppleEvent,AEKeyword theAEKeyword,DescType typeCode,
	Ptr dataPtr,Size dataSize)
	= {0x303C,0x0A0F,0xA816};

pascal OSErr AEPutParamDesc(AppleEvent theAppleEvent,AEKeyword theAEKeyword,AEDesc theAEDesc)
	= {0x303C,0x0610,0xA816};

pascal OSErr AEGetParamPtr(AppleEvent theAppleEvent,AEKeyword theAEKeyword,DescType desiredType,
	DescType *typeCode,Ptr dataPtr,Size maximumSize,Size *actualSize)
	= {0x303C,0x0E11,0xA816};

pascal OSErr AEGetParamDesc(AppleEvent theAppleEvent,AEKeyword theAEKeyword,
    DescType desiredType,AEDesc *result)
    = {0x303C,0x0812,0xA816}; 

pascal OSErr AESizeOfParam(AppleEvent theAppleEvent,AEKeyword theAEKeyword,
    DescType *typeCode,Size *dataSize)
    = {0x303C,0x0829,0xA816}; 

pascal OSErr AEDeleteParam(AppleEvent theAppleEvent,AEKeyword theAEKeyword)
    = {0x303C,0x0413,0xA816}; 

/*--------------------------------------------------------------------------------*/
/*The following calls also apply to type AppleEvent.  Message attributes are far more restricted, and
can only be accessed through the following 5 calls. The various list and record routines cannot be used
to access the attributes of an event. */

pascal OSErr AEGetAttributePtr(AppleEvent theAppleEvent,AEKeyword theAEKeyword,
    DescType desiredType,DescType *typeCode,Ptr dataPtr,Size maximumSize,Size *actualSize)
    = {0x303C,0x0E15,0xA816}; 
pascal OSErr AEGetAttributeDesc(AppleEvent theAppleEvent,AEKeyword theAEKeyword,
    DescType desiredType,AEDesc *result)
    = {0x303C,0x0826,0xA816}; 
pascal OSErr AESizeOfAttribute(AppleEvent theAppleEvent,AEKeyword theAEKeyword,
    DescType *typeCode,Size *dataSize)
    = {0x303C,0x0828,0xA816}; 
pascal OSErr AEPutAttributePtr(AppleEvent theAppleEvent,AEKeyword theAEKeyword,
    DescType typeCode,Ptr dataPtr,Size dataSize)
    = {0x303C,0x0A16,0xA816}; 
pascal OSErr AEPutAttributeDesc(AppleEvent theAppleEvent,AEKeyword theAEKeyword,
    AEDesc theAEDesc)
    = {0x303C,0x0627,0xA816}; 

/*--------------------------------------------------------------------------------*/
/* The next four calls are basic routines used to create, send, and process AppleEvents. */

pascal OSErr AECreateAppleEvent(AEEventClass theAEEventClass,AEEventID theAEEventID,AEAddressDesc
	target,short returnID,long transactionID,AppleEvent *result)
	= {0x303C,0x0B14,0xA816};

pascal OSErr AESend(AppleEvent theAppleEvent,AppleEvent *reply,long sendMode,short sendPriority,
	long timeOutInTicks,IdleProcPtr idleProc,EventFilterProcPtr filterProc)
	= {0x303C,0x0D17,0xA816};

pascal OSErr AEProcessAppleEvent(EventRecord theEventRecord)
	= {0x303C,0x021B,0xA816};

pascal OSErr AEResetTimer(AppleEvent reply)
	= {0x303C,0x0219,0xA816};

/*--------------------------------------------------------------------------------*/
/* The following four calls are available for applications which need more sophisticated control
over when and how events are processed. Applications which implement multi-session servers or 
which implement their own internal event queueing will probably be the major clients of these
routines. */

pascal OSErr AESuspendTheCurrentEvent(AppleEvent theAppleEvent)
	= {0x303C,0x022B,0xA816};

pascal OSErr AEResumeTheCurrentEvent(AppleEvent theAppleEvent,AppleEvent reply,EventHandlerProcPtr dispatcher,
	long handlerRefcon)
	= {0x303C,0x0818,0xA816};

pascal OSErr AEGetTheCurrentEvent(AppleEvent *theAppleEvent)
	= {0x303C,0x021A,0xA816};

pascal OSErr AESetTheCurrentEvent(AppleEvent theAppleEvent)
	= {0x303C,0x022C,0xA816};

/*--------------------------------------------------------------------------------*/
/* The following three calls are used to allow applications to behave courteously 
when a user interaction such as a dialog box is needed. */

pascal OSErr AEGetInteractionAllowed(AEInteractAllowed *level)
	= {0x303C,0x021D,0xA816};

pascal OSErr AESetInteractionAllowed(AEInteractAllowed level)
	= {0x303C,0x011E,0xA816};

pascal OSErr AEInteractWithUser(long timeOutInTicks,QElemPtr nmQReqPtr,IdleProcPtr idleProc)
	= {0x303C,0x061C,0xA816};

/*--------------------------------------------------------------------------------*/
/* These calls are used to set up and modify the event dispatch table */

pascal OSErr AEInstallEventHandler(AEEventClass theAEEventClass,AEEventID theAEEventID,
	EventHandlerProcPtr handler,long handlerRefcon,Boolean isSysHandler)
	= {0x303C,0x091F,0xA816};

pascal OSErr AERemoveEventHandler(AEEventClass theAEEventClass,AEEventID theAEEventID,
	EventHandlerProcPtr handler,Boolean isSysHandler)
	= {0x303C,0x0720,0xA816};

pascal OSErr AEGetEventHandler(AEEventClass theAEEventClass,AEEventID theAEEventID,
	EventHandlerProcPtr *handler,long *handlerRefcon,Boolean isSysHandler)
	= {0x303C,0x0921,0xA816};

/*--------------------------------------------------------------------------------*/
/* These calls are used to set up and modify the event coercion table */

pascal OSErr AEInstallCoercionHandler(DescType fromType,DescType toType,ProcPtr handler,
	long handlerRefcon,Boolean fromTypeIsDesc,Boolean isSysHandler)
	= {0x303C,0x0A22,0xA816};

pascal OSErr AERemoveCoercionHandler(DescType fromType,DescType toType,ProcPtr handler,
	Boolean isSysHandler)
	= {0x303C,0x0723,0xA816};

pascal OSErr AEGetCoercionHandler(DescType fromType,DescType toType,ProcPtr *handler,
	long *handlerRefcon,Boolean *fromTypeIsDesc,Boolean isSysHandler)
	= {0x303C,0x0B24,0xA816};

/*--------------------------------------------------------------------------------*/
/* These calls are used to set up and modify special hooks into the AppleEvent Manager.
At present, only a special preprocessing hook, useful mainly for testing and multi-
threaded applications, is implemented. */

pascal OSErr AEInstallSpecialHandler(AEKeyword functionClass,ProcPtr handler,
	Boolean isSysHandler)
    = {0x303C,0x0500,0xA816}; 
pascal OSErr AERemoveSpecialHandler(AEKeyword functionClass,ProcPtr handler,
	Boolean isSysHandler)
    = {0x303C,0x0501,0xA816}; 
pascal OSErr AEGetSpecialHandler(AEKeyword functionClass,ProcPtr *handler,
	Boolean isSysHandler)
    = {0x303C,0x052C,0xA816}; 


#ifdef __cplusplus
}
#endif

#endif