/************************************************************

Created: Thursday, October 19, 1989 at 11:10 PM
	Balloons.h
	C Interface to the Macintosh Libraries


	Copyright Apple Computer, Inc. 1990
	All rights reserved


************************************************************/

#ifndef __BALLOONS__
#define __BALLOONS__

#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __QUICKDRAW__
#include <Quickdraw.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __TRAPS__
#include <Traps.h>
#endif


#define hmBalloonHelpVersion	1

	/* Help Mgr error range: -850 to -874 */		
#define hmHelpDisabled			-850	/* Show Balloons mode was off, call to routine ignored */
#define hmBalloonAborted		-853	/* Returned if mouse was moving or mouse wasn't in window port rect */
#define hmSameAsLastBalloon 	-854 	/* Returned from HMShowMenuBalloon if menu & item is same as last time */
#define hmHelpManagerNotInited	-855	/* Returned from HMGetHelpMenuHandle if help menu not setup */
#define hmSkippedBalloon		-857	/* Returned from calls if helpmsg specified a skip balloon */
#define hmWrongVersion			-858	/* Returned if help mgr resource was the wrong version */
#define hmUnknownHelpType		-859	/* Returned if help msg record contained a bad type */
#define hmOperationUnsupported	-861	/* Returned from HMShowBalloon call if bad method passed to routine */
#define hmNoBalloonUp			-862	/* Returned from HMRemoveBalloon if no balloon was visible when call was made */
#define hmCloseViewActive		-863	/* Returned from HMRemoveBalloon if CloseView was active */

#define kHMHelpMenuID			-16490	/* Resource ID and menu ID of help menu */
#define kHMAboutHelpItem		1		/* help menu item number of About Balloon Help */
#define kHMShowBalloonsItem		3		/* help menu item number of Show/Hide Balloons */
	
#define kHMHelpID				-5696	/* ID of various Help Mgr package resources (in Pack14 range) */
#define kBalloonWDEFID			126		/* Resource ID of the WDEF proc used in standard balloons */
						
	/* Dialog item template type */
#define helpItem				1		/* value in DITL template that corresponds to the help item */

	/* Options for Help Manager resources in 'hmnu', 'hdlg', 'hrct', 'hovr', & 'hfdr' resources */
#define hmDefaultOptions		0		/* default options for help manager resources */
#define hmUseSubID				1		/* treat resID's in resources as subID's of driver base ID (for Desk Accessories) */
#define hmAbsoluteCoords		2		/* ignore window port origin and treat rectangles as absolute coords (local to window) */ 
#define hmSaveBitsNoWindow		4		/* don't create a window, just blast bits on screen. No update event is generated */
#define hmSaveBitsWindow		8		/* create a window, but restore bits behind window when window goes away & generate update event */
#define hmMatchInTitle			16		/* for hwin resources, match string anywhere in window title string */

	/* Constants for Help Types in 'hmnu', 'hdlg', 'hrct' resources */
#define kHMStringItem			1		/* use a PString in resource */
#define kHMPictItem				2		/* use a 'PICT' ResID in help message or resource */
#define kHMStringResItem 		3		/* use a 'STR#' ResID & index in message or resource */
#define kHMTEResItem 			6		/* use a Styled Text Edit ResID in resource ('TEXT' & 'styl') */
#define kHMSTRResItem			7		/* use a 'STR ' ResID in help message or resource */
#define kHMSkipItem				256		/* don't display a balloon */
#define kHMCompareItem			512		/* Compare PString in menu item w/ PString in resource item ('hmnu' only) */
#define kHMNamedResourceItem	1024	/* Use PString in menu item to get 'STR#', 'PICT', or 'STR ' resource ('hmnu' only) */
#define kHMTrackCntlItem 		2048	/* Reserved */
		
	/* Constants for hmmHelpType's when filling out HMMessageRecord */
#define khmmString				1		/* help message contains a PString */
#define khmmPict				2		/* help message contains a resource ID to a 'PICT' resource */
#define khmmStringRes			3		/* help message contains a res ID & index to a 'STR#' resource */
#define khmmTEHandle			4		/* help message contains a Text Edit handle */
#define khmmPicHandle			5		/* help message contains a Picture handle */
#define khmmTERes				6		/* help message contains a res ID to 'TEXT' & 'styl' resources */
#define khmmSTRRes				7		/* help message contains a res ID to a 'STR ' resource */
		
	/* ResTypes for Styled TE Handles when extracting from Resources */
#define kHMTETextResType 		'TEXT'	/* Resource Type of text data for styled TE record w/o style info */
#define kHMTEStyleResType		'styl'	/* Resource Type of style information for styled TE record */

	/* Generic defines for the state parameter used when extracting 'hmnu' & 'hdlg' messages*/
#define kHMEnabledItem			0		/* item is enabled, but not checked or control value = 0 */
#define kHMDisabledItem 		1		/* item is disabled, grayed in menus or disabled in dialogs */
#define kHMCheckedItem			2		/* item is enabled, and checked or control value = 1 */
#define kHMOtherItem			3		/* item is enabled, and control value > 1 */
		
	/* well known help res types */
#define kHMMenuResType			'hmnu'	/* ResType of help resource for supporting menus */
#define kHMDialogResType		'hdlg'	/* ResType of help resource for supporting dialogs */
#define kHMWindListResType		'hwin'	/* ResType of help resource for supporting windows */
#define kHMRectListResType		'hrct'	/* ResType of help resource for rectangles in windows */
#define kHMOverrideResType		'hovr'	/* ResType of help resource for overriding system balloons */
#define kHMFinderApplResType	'hfdr'	/* ResType of help resource for custom balloon in Finder */

	/* constants to pass to method parameter in HMShowBalloon */
#define kHMRegularWindow		0		/* Create a regular window floating above all windows */
#define kHMSaveBitsNoWindow		1		/* Just save the bits and draw (for MDEF calls */
#define kHMSaveBitsWindow		2		/* Regular window, save bits behind, AND generate update event */

typedef	struct
{
	short	hmmResID;
	short	hmmIndex;
} HMStringResType;

struct HMMessageRecord {
		short		hmmHelpType;
		union {
				char				hmmString[256];
				short				hmmPict;
				Handle				hmmTEHandle;
				HMStringResType		hmmStringRes;
				short				hmmPictRes;
				Handle				hmmPictHandle;
				short				hmmTERes;
				short				hmmSTRRes;
			} u;
};

typedef struct HMMessageRecord HMMessageRecord;
typedef HMMessageRecord *HMMessageRecPtr;


#ifdef __cplusplus
extern "C" {
#endif

pascal	short HMGetHelpMenuHandle(MenuHandle *mh)
		= {0x303C,0x0200,_Pack14};

pascal	short HMShowBalloon(	HMMessageRecord aHelpMsg,
								Point tip,
								Rect* alternateRect,
								Ptr tipProc,
								short theProc, short variant,
								short method)				
		= {0x303C,0x0B01,_Pack14};
				
pascal	short HMRemoveBalloon(void)
		= {0x303C,0x0002,_Pack14};
		
pascal	Boolean HMGetBalloons(void)
		= {0x303C,0x0003,_Pack14};					/* note: doesn't load package! */
		
pascal	short HMSetBalloons(Boolean flag)
		= {0x303C,0x0104,_Pack14};
		
pascal	short HMShowMenuBalloon(short itemNum, short itemMenuID,
								long itemFlags, long itemReserved,
								Point tip,
								Rect* alternateRect,
								Ptr	tipProc,
								short theProc,short variant)
		= {0x303C,0x0E05,_Pack14};

pascal	short HMGetIndHelpMsg(ResType whichType,
								short whichResID, short whichMsg, short whichState,
								long *options,
								Point *tip,
								Rect* altRect,
								short *theProc, short *variant,
								HMMessageRecord *aHelpMsg,
								short *count)
		= {0x303C,0x1306,_Pack14};
		
pascal	Boolean HMIsBalloon(void)
		= {0x303C,0x0007,_Pack14};
		
pascal	short HMSetFont(short font)
		= {0x303C,0x0108,_Pack14};

pascal	short HMSetFontSize(short fontSize)
		= {0x303C,0x0109,_Pack14};

pascal	short HMGetFont(short *font)
		= {0x303C,0x020A,_Pack14};
		
pascal	short HMGetFontSize(short *fontSize)
		= {0x303C,0x020B,_Pack14};

pascal	short HMSetDialogResID(short resID)
		= {0x303C,0x010C,_Pack14};

pascal	short HMSetMenuResID(short menuID, short resID)
		= {0x303C,0x020D,_Pack14};

pascal	short HMBalloonRect(HMMessageRecord aHelpMsg,
								Rect *coolRect)
		= {0x303C,0x040E,_Pack14};				

pascal	short HMBalloonPict(HMMessageRecord aHelpMsg,
								Handle *coolPict)
		= {0x303C,0x040F,_Pack14};	
		
pascal	short HMScanTemplateItems(short whichID, short whichResFile,
								ResType whichType)
		= {0x303C,0x0410,_Pack14};	
	
		
pascal	short HMExtractHelpMsg(ResType whichType, short whichResID, short whichMsg,
								short whichState,
								HMMessageRecord *aHelpMsg)
		= {0x303C,0x0711,_Pack14};	

pascal	short HMFillCitationString(StringPtr origString, StringPtr stuffString,
								short key)
		= {0x303C,0x0512,_Pack14};	

pascal	short HMGetDialogResID(short *resID)
		= {0x303C,0x0213,_Pack14};

pascal	short HMGetMenuResID(short menuID,short *resID)
		= {0x303C,0x0314,_Pack14};

pascal	short HMGetBalloonWindow(WindowPtr *window)
		= {0x303C,0x0215,_Pack14};
		
#ifdef __cplusplus
}
#endif

#endif
