{
Created: Wednesday, June 27, 1990 at 6:42 PM
    SoundInput.p
    Pascal Interface to the Macintosh Libraries

        Copyright Apple Computer, Inc. 1989-1990
        All rights reserved
}


{$IFC UNDEFINED UsingIncludes}
{$SETC UsingIncludes := 0}
{$ENDC}

{$IFC NOT UsingIncludes}
    UNIT SoundInput;
    INTERFACE
{$ENDC}

{$IFC UNDEFINED UsingSoundInput}
{$SETC UsingSoundInput := 1}

{$I+}
{$SETC SoundInputIncludes := UsingIncludes}
{$SETC UsingIncludes := 1}
{$IFC UNDEFINED UsingTypes}
{$I $$Shell(PInterfaces)Types.p}
{$ENDC}
{$SETC UsingIncludes := SoundInputIncludes}


CONST

{Error codes}

siNoSoundInHardware	=	-220;			{  not used anywhere! }
siBadSoundInDevice =	-221;			{ invalid index passed to SoundInGetIndexedDevice }
siNoBufferSpecified	=	-222;			{ returned by synchronous SPBRecord if nil buffer passed }
siInvalidCompression =	-223;			{ invalid compression type }
siHardDriveTooSlow =	-224;			{ hard drive too slow to record to disk }
siInvalidSampleRate =	-225;			{ invalid sample rate }
siInvalidSampleSize =	-226;			{ invalid sample size }
siDeviceBusyErr =		-227;			{ input device already in use }
siBadDeviceName =		-228;			{ input device could not be opened }
siBadRefNum =			-229;			{ invalid input device reference number }
siInputDeviceErr =		-230;			{ input device hardware failure }
siUnknownInfoType =		-231;			{ invalid info type selector (returned by driver) }
siUnknownQuality =		-232;			{ invalid quality selector (returned by driver) }

siDeviceIsConnected =	 1;				{ input device is connected and ready for input }
siDeviceNotConnected =	 0;				{ input device is not connected }
siDontKnowIfConnected =	-1;				{ can't tell if input device is connected }

siReadPermission =		 0;				{ permission passed to SPBOpenDevice }
siWritePermission = 	 1;				{ permission passed to SPBOpenDevice }

{ Info Selectors for Sound Input Drivers }

siDeviceConnected =		'dcon';			{ input device connection status }
siRTFOnOff =			'rtf ';			{ record to file state }
siAGCOnOff =			'agc ';			{ automatic gain control state }
siPlayThruOnOff	=		'plth';			{ playthrough state }
siTwosComplementOnOff =	'twos';			{ two's complement state }
siLevelMeterOnOff =		'lmet';			{ level meter state }
siPauseRecording =		'paus';			{ recording paused state }
siRecordingQuality =	'qual';			{ recording quality }
siVoxRecordInfo	=		'voxr';			{ VOX record parameters }
siVoxStopInfo =			'voxs';			{ VOX stop parameters }
siNumberChannels =		'chan';			{ current number of channels }
siSampleSize =			'ssiz';			{ current sample size }
siSampleRate =			'srat';			{ current sample rate }
siCompressionType =		'comp';			{ current compression type }
siCompressionFactor	=	'cmfa';			{ current compression factor }
siCompressionHeader	=	'cmhd';			{ return compression header }
siUserInterruptProc	=	'user';			{ user interrupt routine }
siDeviceName =			'name';			{ input device name }
siDeviceIcon =			'icon';			{ input device icon }
siDeviceBufferInfo =	'dbin';			{ size of interrupt buffer }
siSampleSizeAvailable =	'ssav';			{ sample sizes available }
siSampleRateAvailable =	'srav';			{ sample rates available }
siCompressionAvailable ='cmav';			{ compression types available }
siChannelAvailable =	'chav';			{ number of channels available }
siInitializeDriver =	'init';			{ initialize driver }
siCloseDriver =			'clos';			{ close driver }
siAsync	=				'asyn';			{ asynchronous capability }
siOptionsDialog =		'optd';			{ display options dialog }

{ Qualities }

siBestQuality =			'best';
siBetterQuality =		'betr';
siGoodQuality =			'good';


{ Function selectors for the Sound Input calls }

siToolNum =				20;				{ tool number of Sound Input Manager }
siMOVEL = 				$203C;			{ MOVEI.L	#selector,D0 }

siVersion =				(0*256)+0;
siSndRecord =			(8*256)+4;
siSndRecordToFile =		(7*256)+8;
siSignInDevice =		(3*256)+12;
siSignOutDevice =		(1*256)+16;
siGetIndexedDevice =	(5*256)+20;
siOpenDevice =			(5*256)+24;
siCloseDevice =			(2*256)+28;
siRecord =				(3*256)+32;
siRecordToFile =		(4*256)+36;
siPauseRecordingR =		(2*256)+40;
siResumeRecording =		(2*256)+44;
siStopRecording	 =		(2*256)+48;
siGetRecordingStatus =	(14*256)+52;
siGetDeviceInfo	 =		(6*256)+56;
siSetDeviceInfo	 =		(6*256)+60;
siMilliSecondsToBytes =	(4*256)+64;
siBytesToMilliSeconds =	(4*256)+68;
siSetupSndHeader =		(13*256)+72;
siSetupAIFFHeader =		(11*256)+76;


TYPE

SPBPtr = ^SPB;
SPB = RECORD
	inRefNum:			LONGINT;			{ reference number of sound input device }
	count:				LONGINT;			{ number of bytes to record }
	milliseconds:		LONGINT;			{ number of milliseconds to record }
	bufferLength:		LONGINT;			{ length of buffer in bytes }
	bufferPtr:			Ptr;				{ buffer to store sound data in }
	completionRoutine:	ProcPtr;			{ completion routine }
	interruptRoutine:	ProcPtr;			{ interrupt routine }
	userLong:			LONGINT;			{ user-defined field }
	error:				OSErr;				{ error }
	unused1:			LONGINT;			{ reserved - must be zero }
	END;


FUNCTION SPBVersion : LONGINT;
	INLINE siMOVEL,siVersion,siToolNum,$A800;

FUNCTION SndRecord(filterProc: ProcPtr; corner: Point; quality: OSType;
	VAR sndHandle: Handle) : OSErr;
	INLINE siMOVEL,siSndRecord,siToolNum,$A800;

FUNCTION SndRecordToFile(filterProc: ProcPtr; corner: Point; quality: OSType;
	fRefNum: INTEGER) : OSErr;
	INLINE siMOVEL,siSndRecordToFile,siToolNum,$A800;

FUNCTION SPBSignInDevice(deviceRefNum: INTEGER; deviceName: Str255) : OSErr;
	INLINE siMOVEL,siSignInDevice,siToolNum,$A800;

FUNCTION SPBSignOutDevice(deviceRefNum: INTEGER) : OSErr;
	INLINE siMOVEL,siSignOutDevice,siToolNum,$A800;

FUNCTION SPBGetIndexedDevice(count: INTEGER; VAR deviceName: Str255;
	VAR deviceIconHandle: Handle) : OSErr;
	INLINE siMOVEL,siGetIndexedDevice,siToolNum,$A800;

FUNCTION SPBOpenDevice(deviceName: Str255; permission: INTEGER; VAR inRefNum: LONGINT) : OSErr;
	INLINE siMOVEL,siOpenDevice,siToolNum,$A800;

FUNCTION SPBCloseDevice(inRefNum: LONGINT) : OSErr;
	INLINE siMOVEL,siCloseDevice,siToolNum,$A800;

FUNCTION SPBRecord(inParamPtr: SPBPtr; asynchFlag: BOOLEAN) : OSErr;
	INLINE siMOVEL,siRecord,siToolNum,$A800;

FUNCTION SPBRecordToFile(fRefNum: INTEGER; inParamPtr: SPBPtr; asynchFlag: BOOLEAN) : OSErr;
	INLINE siMOVEL,siRecordToFile,siToolNum,$A800;

FUNCTION SPBPauseRecording(inRefNum: LONGINT) : OSErr;
	INLINE siMOVEL,siPauseRecordingR,siToolNum,$A800;

FUNCTION SPBResumeRecording(inRefNum: LONGINT) : OSErr;
	INLINE siMOVEL,siResumeRecording,siToolNum,$A800;

FUNCTION SPBStopRecording(inRefNum: LONGINT) : OSErr;
	INLINE siMOVEL,siStopRecording,siToolNum,$A800;

FUNCTION SPBGetRecordingStatus(inRefNum: LONGINT; VAR recordingStatus: INTEGER; VAR meterLevel: INTEGER;
	VAR totalSamplesToRecord: LONGINT; VAR numberOfSamplesRecorded: LONGINT;
	VAR totalMsecsToRecord: LONGINT; VAR numberOfMsecsRecorded: LONGINT) : OSErr;
	INLINE siMOVEL,siGetRecordingStatus,siToolNum,$A800;

FUNCTION SPBGetDeviceInfo(inRefNum: LONGINT; infoType: OSType; infoData: Ptr) : OSErr;
	INLINE siMOVEL,siGetDeviceInfo,siToolNum,$A800;

FUNCTION SPBSetDeviceInfo(inRefNum: LONGINT; infoType: OSType; infoData: Ptr) : OSErr;
	INLINE siMOVEL,siSetDeviceInfo,siToolNum,$A800;

FUNCTION SPBMilliSecondsToBytes(inRefNum: LONGINT; VAR milliSeconds: LONGINT) : OSErr;
	INLINE siMOVEL,siMilliSecondsToBytes,siToolNum,$A800;

FUNCTION SPBBytesToMilliSeconds(inRefNum: LONGINT; VAR byteCount: LONGINT) : OSErr;
	INLINE siMOVEL,siBytesToMilliSeconds,siToolNum,$A800;

FUNCTION SetupSndHeader(sndHandle: Handle; numChannels: INTEGER; sampleRate: Fixed;
				sampleSize: INTEGER; compressionType: OSType; baseNote: INTEGER;
				numBytes: LONGINT; VAR headerLen: INTEGER) : OSErr;
	INLINE siMOVEL,siSetupSndHeader,siToolNum,$A800;

FUNCTION SetupAIFFHeader(fRefNum: INTEGER;  numChannels: INTEGER; sampleRate: Fixed;
				sampleSize: INTEGER; compressionType: OSType;
				numBytes: LONGINT; numFrames: LONGINT) : OSErr;
	INLINE siMOVEL,siSetupAIFFHeader,siToolNum,$A800;

{$ENDC}    { UsingSoundInput }

{$IFC NOT UsingIncludes}
	END.
{$ENDC}
